@@ -1,20 +1,24 @@
 package CPAN::Releases::Latest;
-$CPAN::Releases::Latest::VERSION = '0.05';
+$CPAN::Releases::Latest::VERSION = '0.06';
 use 5.006;
 use Moo;
 use File::HomeDir;
 use File::Spec::Functions 'catfile';
 use MetaCPAN::Client 1.001001;
+use Module::Runtime qw/ is_module_name require_module /;
 use CPAN::DistnameInfo;
 use Carp;
 use autodie;
 
 my $FORMAT_REVISION = 1;
 
-has 'max_age'    => (is => 'ro', default => sub { '1 day' });
-has 'cache_path' => (is => 'rw');
-has 'basename'   => (is => 'ro', default => sub { 'latest-releases.txt' });
-has 'path'       => (is => 'ro');
+has 'max_age'           => (is => 'ro', default => sub { '1 day' });
+has 'cache_path'        => (is => 'rw');
+has 'basename'          => (is => 'ro', default => sub { 'latest-releases.txt' });
+has 'path'              => (is => 'ro');
+has 'source'            => (is => 'ro', default => sub { 'MetaCPAN' });
+
+has '_indexer'          => (is => 'lazy');
 
 sub BUILD
 {
@@ -54,57 +58,25 @@ sub BUILD
 sub _build_cached_index
 {
     my $self     = shift;
-    my $distdata = $self->_get_release_info_from_metacpan();
+    my $indexer  = $self->_indexer;
+    my $distdata = $indexer->get_release_info();
 
     $self->_write_cache_file($distdata);
 }
 
-sub _get_release_info_from_metacpan
+sub _build__indexer
 {
-    my $self       = shift;
-    my $client     = MetaCPAN::Client->new();
-    my $query      = {
-                        either => [
-                                      { all => [
-                                          { status   => 'latest'    },
-                                          { maturity => 'released'  },
-                                      ]},
+    my $self             = shift;
+    my $base_module_name = $self->source;
 
-                                      { all => [
-                                          { status   => 'cpan'      },
-                                          { maturity => 'developer' },
-                                      ]},
-                                   ]
-                     };
-    my $params     = {
-                         fields => [qw(name version date status maturity stat download_url)]
-                     };
-    my $result_set = $client->release($query, $params);
-    my $distdata   = {
-                         released  => {},
-                         developer => {},
-                     };
-
-    while (my $release = $result_set->next) {
-        my $maturity = $release->maturity;
-        my $slice    = $distdata->{$maturity};
-        my $path     = $release->download_url;
-           $path     =~ s!^.*/authors/id/!!;
-        my $distinfo = CPAN::DistnameInfo->new($path);
-        my $distname = defined($distinfo) && defined($distinfo->dist)
-                       ? $distinfo->dist
-                       : $release->name;
-
-        next unless !exists($slice->{ $distname })
-                 || $release->stat->{mtime} > $slice->{$distname}->{time};
-        $slice->{ $distname } = {
-                                    path => $path,
-                                    time => $release->stat->{mtime},
-                                    size => $release->stat->{size},
-                                };
+    if (not is_module_name($base_module_name)) {
+        croak "source '$base_module_name' is not a valid module name";
     }
 
-    return $distdata;
+    my $full_class_name  = "CPAN::Releases::Latest::Source::$base_module_name";
+    require_module($full_class_name);
+
+    return $full_class_name->new();
 }
 
 sub _write_cache_file
@@ -154,6 +126,17 @@ sub release_iterator
     return CPAN::Releases::Latest::ReleaseIterator->new( latest => $self, @_ );
 }
 
+sub distribution_iterator
+{
+    my $self = shift;
+
+    require CPAN::Releases::Latest::DistributionIterator;
+    return CPAN::Releases::Latest::DistributionIterator->new(
+                latest => $self,
+                @_
+           );
+}
+
 sub _open_file
 {
     my $self       = shift;
@@ -226,13 +209,14 @@ CPAN::Releases::Latest - find latest release(s) of all dists on CPAN, including
 
 =head1 DESCRIPTION
 
-VERY MUCH AN ALPHA. ALL THINGS MAY CHANGE.
-
-This module uses the MetaCPAN API to construct a list of all dists on CPAN.
+This module constructs a list of all dists on CPAN, by default using the MetaCPAN API.
 The generated index is cached locally.
-It will let you iterate across these, returning the latest release of the dist.
-If the latest release is a developer release, then you'll first get back the
-non-developer release (if there is one), and then you'll get back the developer release.
+It will let you iterate over the index, either release by release,
+or distribution by distribution.
+
+See below for details of the two iterators you can instantiate.
+
+B<Note:> this is very much an alpha release; all things may change.
 
 When you instantiate this class, you can specify the C<max_age> of
 the generated index. You can specify the age
@@ -250,6 +234,107 @@ If you already have a cached copy of the index, and it is less than
 the specified age, then we'll use your cached copy and not even
 check with MetaCPAN.
 
+=head2 distribution_iterator
+
+The C<distribution_iterator> method returns an iterator which
+will process the index dist by dist:
+
+ my $latest   = CPAN::Releases::Latest->new();
+ my $iterator = $latest->distribution_iterator();
+
+ while (my $dist = $iterator->next_distribution) {
+    print $dist->distname, "\n";
+    process_release($dist->release);
+    process_release($dist->developer_release);
+ }
+
+The iterator returns instances of L<CPAN::Releases::Latest::Distribution>,
+or C<undef> when the index has been exhausted.
+The distribution object has three attributes:
+
+=over 4
+
+=item * distname: the distribution name as determined by L<CPAN::DistnameInfo>
+
+=item * release: a release object for the latest non-developer release, or C<undef>
+
+=item * developer_release: a release object for the latest developer release that is more recent than the latest non-developer release, or C<undef>
+
+=back
+
+The release objects are instances of L<CPAN::Releases::Latest::Release>,
+which are described in the next section, below.
+
+=head2 release_iterator
+
+The C<release_iterator> method returns an iterator which will process the index
+release by release. See the example in the SYNOPSIS.
+
+You will see the releases ordered distribution by distribution.
+For a given distribution you'll first see the latest non-developer release,
+if there is one;
+if the most recent release for the distribution is a developer release,
+then you'll see that.
+So for any dist you'll see at most two releases, and the developer release
+will always come second.
+
+The release objects are instances of L<CPAN::Releases::Latest::Release>,
+which have the following attributes:
+
+=over 4
+
+=item * distname: the distribution name as determined by L<CPAN::DistnameInfo>
+
+=item * path: the partial path for the release tarball (eg C<N/NE/NEILB/enum-1.05.tar.gz>)
+
+=item * timestamp: an epoch-based timestamp for when the tarball was uploaded to PAUSE.
+
+=item * size: the size of the release tarball, in bytes.
+
+=item * distinfo: an instance of L<CPAN::DistnameInfo>, which is constructed lazily.
+
+=back
+
+=head1 Data source
+
+By default the locally cached index is generated using information requested
+from MetaCPAN, using L<MetaCPAN::Client>. The plugin which does this is
+L<CPAN::Releases::Latest::Source::MetaCPAN>. You can explicitly specify
+the source when calling the constructor:
+
+ $latest = CPAN::Releases::Latest->new( source => 'MetaCPAN' );
+
+You can use a different source for the data, by providing your own plugin,
+which must live in the C<CPAN::Releases::Latest::Source> namespace.
+
+The plugin must return a hashref that has the following structure:
+
+ {
+   release => {
+
+     'Graph' => {
+        path => 'J/JH/JHI/Graph-0.96.tar.gz',
+        time => 1369483123,
+        size => 147629,
+     },
+
+   },
+
+   developer => {
+
+     'Graph' => {
+        path => 'N/NE/NEILB/Graph-0.96_01.tar.gz',
+        time => 1394362358,
+        size => 147335,
+     },
+
+   }
+
+ }
+
+At the moment this isn't enforced, but a future version will croak
+if the source doesn't return the right structure.
+
 =head1 SEE ALSO
 
 L<CPAN::ReleaseHistory> provides a similar iterator, but for all releases
